#region Copyright Syncfusion Inc. 2001 - 2017
// Copyright Syncfusion Inc. 2001 - 2017. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Caliburn.Micro;
using System.Windows.Controls;
using System.ComponentModel.Composition.Hosting;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Primitives;

namespace DockingManagerMVVMCaliburnMicro
{
    public class ShellBootstrapper : Bootstrapper<ShellViewModel>
    {
        public ShellBootstrapper()
        {
            ConventionManager.AddElementConvention<DockingAdapterMVVM.DockingAdapter>(DockingAdapterMVVM.DockingAdapter.ItemsSourceProperty, "DataContext", "Loaded");
        }

        private CompositionContainer container;

        protected override void Configure()
        {
         //An aggregate catalog that combines multiple catalogs  
         var catalog = new AggregateCatalog();

         //Adds all the parts found in the same assembly as the Program class  
         catalog.Catalogs.Add(new AssemblyCatalog(typeof(ShellBootstrapper).Assembly));

         //Create the CompositionContainer with the parts in the catalog  
         container = new CompositionContainer(catalog);

         var batch = new CompositionBatch();
         batch.AddExportedValue<IWindowManager>(new WindowManager());
         batch.AddExportedValue<IEventAggregator>(new EventAggregator());
         batch.AddExportedValue(container);

         container.Compose(batch);
      }

        protected override object GetInstance(Type serviceType, string key)
        {
            string contract = string.IsNullOrEmpty(key) ? AttributedModelServices.GetContractName(serviceType) : key;
            var exports = container.GetExportedValues<object>(contract);

            if (exports.Any())
                return exports.First();

            throw new Exception(string.Format("Could not locate any instances of contract {0}.", contract));
        }

        protected override IEnumerable<object> GetAllInstances(Type serviceType)
        {
            return container.GetExportedValues<object>(AttributedModelServices.GetContractName(serviceType));
        }

        protected override void BuildUp(object instance)
        {
            container.SatisfyImportsOnce(instance);
        }
    }
}
